#!/usr/bin/perl
# vim:ts=4:et
#
# (c) Robert Luberda <robert@debian.org>, 2007, GPLv2 or later
#
# Simple script to update the registry files from the install.js 
# provided in the upstream *.xpi files.
# Needs to be run by hand for new upstream versions. 
#

use JavaScript;     # from the libjavascript-perl package

my $UNPACKDIR="debian/BUILD/UNPACK";
my $OUTDIR="debian/registry";

for (my $i = 0; $i < 10; $i++) {
	-d $UNPACKDIR && last;
	chdir "..";
}	

die ("Can't find $UNPACKDIR directory. Please run debian/build.sh to generate it.\n") unless -d $UNPACKDIR;


my @JSOutput = ();
my $success=0;

sub debug {
##    my $msg = shift;
##    my @args = @_;

##    print STDOUT $msg . ($#args > -1 ? ': ' . join (', ', @args) . "" : "") . "\n";
}    

sub JSGetFolder {
    &debug('getFolder', @_);

	my $type	=   shift;
	my $dir	    =   shift;


    $dir = "" unless defined $dir;

    return JSGetFolder("Program", "/chrome") if $type eq "Chrome";
    return "locale,install,url,jar:resource:$dir" if $type eq "Program" or $type eq "Profile";
    return "$type/$dir";
}
 
sub JSRegisterChrome {
    &debug('registerChrome', @_);

    my ($dummy, $folder, $regdir) = @_;

    push(@JSOutput, "$folder!/$regdir");
    return $success;
}    

sub ParseFile {
    my $filename = shift;
    my $runtime = new JavaScript::Runtime();
    my $context = $runtime->create_context();

    @JSOutput = ();
    
    $context->eval("var LOCALE=1;");
    $context->eval("var DELAYED_CHROME=1;");
    $context->eval("var PROFILE_CHROME=1;");
    $context->eval("var PROFILE_CHROME=1;");
    $context->eval("var SUCCESS=" . $success . ';');
    

    foreach my $variable (  'DELAYED_CHROME', 'LOCALE', 'PROFILE_CHROME', 
                            'REBOOT_NEEDED', 'SUCCESS' ) {
        $context->eval("var $variable=$success;");
    }	
	

    foreach my $function  ( 'addDirectory', 'alert', 'cancelInstall', 
                            'initInstall', 'logComment', 'performInstall',
                            'resetError', 'setPackageFolder') {
        $context->bind_function( name => $function, 
                                 func => sub { &debug($function, @_); 
                                               return $success } 
                               );
    }	
    $context->bind_function( name => 'confirm', 
                             func => sub {  &debug('confirm', @_); 
                                            return 1 } 
			   );						    
    $context->bind_function( name => 'loadResources', 
                             func => sub {  &debug('loadResources', @_); 
                                            return undef } 
                           );
    $context->bind_function( name => 'fileGetDiskSpaceAvailable', 
                             func => sub {  &debug('fileGetDiskSpaceAvailable', @_) ; 
                                     return 1000000000 } 
                           );
    $context->bind_function( name => 'registerChrome', 
                             func => \&JSRegisterChrome );
    $context->bind_function( name => 'getFolder', 
                             func => \&JSGetFolder );

    open (SCRIPT, $filename) or die "Can't open $filename: $!\n";
    my @script=<SCRIPT>;
    close SCRIPT;
    
    # I'm too lazy to crete bindings for Install.platform and messages resources
    grep { 	s/Install.platform/"unix"/g; 
	        s/'platform'\s+in\s+Install/true/g;
            s/regionNode/regioncode/g;
	        s/messages\.\w+/'$&'/g
         } @script;

    ## print @script;
    
    debug("Evaluate $filename");
    $context->eval(join("\n", @script));

    my $langcode    =  $context->eval('langcode');
    my $regioncode  =   $context->eval('regioncode');
    $regioncode     = 'GR' if not defined $regioncode and $langcode eq 'el';
    $regioncode     = 'GB' if not defined $regioncode and $langcode eq 'en';
    return $langcode . '-' . $regioncode;



    return . '-' . $context->eval('regioncode')
}   


my @installfiles = glob("$UNPACKDIR/*/install.js");
die ("Can't find any install.js file") if $#installfiles < 0;


my $suff=".backup";
grep { unlink "$_$suff" if -f "$_$suff"; rename "$_", "$_$suff" } glob ("$OUTDIR/*-locale");

foreach my $installfile (@installfiles) {
    print STDOUT "Parse file $installfile\n";
    @JSOutput = ();
    my $lang = &ParseFile($installfile);
    my $regfile = "$OUTDIR/$lang-locale";

    open REGISTRY, ">>$regfile" or die "Can't open $regfile for writing: $!\n";
    print REGISTRY join("\n", @JSOutput) . "\n";
    close REGISTRY;
    print STDOUT "Wrote output to $regfile\n\n";
}    

